#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

// Button Pin Setup 
const int l_up_button = 9; // Left player up button pin
const int l_down_button = 2; // Left player down button pin
const int r_up_button = 3; // Right player up button pin
const int r_down_button = 5; // Right player down button pin
const int led1 = 11; // LED 1 pin
const int led2 = 12; // LED 2 pin

// Screen Size Parameters
int x_pixels = 128;
int y_pixels = 32;

// Paddle Parameters
int paddle_height = 10;
int paddle_width = 3;

int l_paddle_height = paddle_height;
int r_paddle_height = paddle_height;

int speakerPin = 8;

// Ball Parameters
int ball_size = 2; // Dimensiunea bilei
int x_vel = 2;
int y_vel = 2;
int x_pos = 5;
int y_pos = 16;

// Paddle positions:
int l_pos = 0;
int r_pos = 0;

// Player scores
int l_score = 0;
int r_score = 0;

const int max_score = 7; // Punctajul maxim pentru a câștiga
const int max_vel = 6; // Limita maxima pentru viteza

unsigned long previousMillis = 0;
const long interval = 16; // Aproximativ 30 FPS (1000ms / 30)

#if (SSD1306_LCDHEIGHT != 32)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

void setup() { //GPIO
  pinMode(l_up_button, INPUT);
  pinMode(l_down_button, INPUT);
  pinMode(r_up_button, INPUT);
  pinMode(r_down_button, INPUT);
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  Serial.begin(9600); //partea de UART
  randomSeed(analogRead(0));
  //SSD1306_SWITCHCAPVCC - ecranul oled e alimentat prin condensator de comutatie
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C); // initialize with the I2C addr 0x3C (for the 128x32)

  // Show splashscreen
  display.display();
  display.clearDisplay();

  // Display Arduino Pong splashscreen
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(display.width() / 2 - 20, 0);
  display.println("Arduino");
  display.setCursor(display.width() / 2 - 10, 8);
  display.println("Pong");
  display.display();
  delay(2000);
}

void loop() { //timere
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis; //timpul de la ultima actualizare a jocului

    // Update game logic
    updateGame();

    // Draw game directly in the loop
    drawGame();
  }
}

void updateGame() {
  // Check if a player has won
  if (l_score >= max_score) {
    announce_winner(led1);
    reset_game();
    return;
  } else if (r_score >= max_score) {
    announce_winner(led2);
    reset_game();
    return;
  }
  //Intreruperi - button handling
  // Update position of left paddle: 
  if (digitalRead(l_up_button) && l_pos > 0) { l_pos -= 2; } // Viteza mărită
  else if (digitalRead(l_down_button) && l_pos < (y_pixels - l_paddle_height)) { l_pos += 2; } // Viteza mărită
  
  // Update position of right paddle:
  if (digitalRead(r_up_button) && r_pos > 0) { r_pos -= 2; } // Viteza mărită
  else if (digitalRead(r_down_button) && r_pos < (y_pixels - r_paddle_height)) { r_pos += 2; } // Viteza mărită

  // Update ball position:
  x_pos += x_vel;
  y_pos += y_vel;

  // Check for ball bouncing off ceiling:
  if (y_pos >= y_pixels - ball_size || y_pos <= 0) { y_vel = -y_vel; adjust_speed(); }

  // Check for ball bouncing off paddles:
  if (ball_on_right_paddle()) {
    x_vel = -x_vel;
    adjust_speed();
    tone(speakerPin, 300, 100);
  } else if (ball_on_left_paddle()) {
    x_vel = -x_vel;
    adjust_speed();
    tone(speakerPin, 250, 100);
  }

  // Check for ball hitting a wall:
  if (x_pos >= x_pixels - ball_size) {
    ball_reset(false);
    l_score += 1;
    tone(speakerPin, 50, 100);
    digitalWrite(led1, HIGH); // Aprinde LED 1
    delay(500); // Așteaptă 500 ms
    digitalWrite(led1, LOW); // Stinge LED 1
  } else if (x_pos <= 0) {
    ball_reset(true);
    r_score += 1;
    tone(speakerPin, 50, 100);
    digitalWrite(led2, HIGH); // Aprinde LED 2
    delay(500); // Așteaptă 500 ms
    digitalWrite(led2, LOW); // Stinge LED 2
  }
}

void drawGame() {  //partea de I2C
  display.clearDisplay();
  display.fillRect(x_pos, y_pos, ball_size, ball_size, WHITE);
  display.fillRect(0, l_pos, paddle_width, l_paddle_height, WHITE);
  display.fillRect(x_pixels - paddle_width, r_pos, paddle_width, r_paddle_height, WHITE);

  // Display scores
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(display.width() / 4, 0);
  display.println(l_score);
  display.setCursor(display.width() * 3 / 4, 0);
  display.println(r_score);

  // Display all elements
  display.display();
}

bool ball_on_right_paddle() {
  return ((x_pos + x_vel) >= x_pixels - paddle_width - ball_size && y_pos + ball_size >= r_pos && y_pos <= (r_pos + r_paddle_height));
}

bool ball_on_left_paddle() {
  return ((x_pos + x_vel) <= paddle_width && y_pos + ball_size >= l_pos && y_pos <= (l_pos + l_paddle_height));
}

void ball_reset(bool left) {
  y_pos = random(display.height() - ball_size);
  y_vel = random(2) ? 3 : -3;

  if (left) {
    x_vel = 2;
    x_pos = paddle_width;
  } else {
    x_vel = -2;
    x_pos = display.width() - paddle_width - ball_size;
  }

  adjust_paddles();
}

void adjust_speed() {
  if (x_vel > 0 && x_vel < max_vel) {
    x_vel += 1;
  } else if (x_vel < 0 && x_vel > -max_vel) {
    x_vel -= 1;
  }
  
  if (y_vel > 0 && y_vel < max_vel) {
    y_vel += 1;
  } else if (y_vel < 0 && y_vel > -max_vel) {
    y_vel -= 1;
  }
}

void adjust_paddles() {
  int new_height = random(5, 15); // Dimensiunea noua a paletei intre 5 si 15 pixeli
  int mode = random(3); // 0 = ambele palete egale, 1 = stânga mai mică, 2 = dreapta mai mică

  if (mode == 0) {
    l_paddle_height = new_height;
    r_paddle_height = new_height;
  } else if (mode == 1) {
    l_paddle_height = new_height;
    r_paddle_height = paddle_height;
  } else {
    l_paddle_height = paddle_height;
    r_paddle_height = new_height;
  }
}

void announce_winner(int winner_led) {
  for (int i = 0; i < 6; i++) { // Clipsește LED-ul de 6 ori
    digitalWrite(winner_led, HIGH);
    delay(250);
    digitalWrite(winner_led, LOW);
    delay(250);
  }
}

void reset_game() {
  l_score = 0;
  r_score = 0;
  l_paddle_height = 10;
  r_paddle_height = 10;
  ball_reset(random(2)); // Resetează mingea aleatoriu din stânga sau dreapta
}
